@@ -1,6 +1,6 @@
 #!/bin/sh
-#Copyright (C) The openNDS Contributors 2004-2021
-#Copyright (C) BlueWave Projects and Services 2015-2021
+#Copyright (C) The openNDS Contributors 2004-2022
+#Copyright (C) BlueWave Projects and Services 2015-2022
 #This software is released under the GNU GPL license.
 #
 # Warning - shebang sh is for compatibliity with busybox ash (eg on OpenWrt)
@@ -200,6 +200,7 @@ thankyou_page () {
 
 landing_page() {
 	originurl=$(printf "${originurl//%/\\x}")
+	gatewayurl=$(printf "${gatewayurl//%/\\x}")
 
 	# Add the user credentials to $userinfo for the log
 	userinfo="$userinfo, user=$username, email=$emailaddress"
@@ -223,14 +224,13 @@ landing_page() {
 			<italic-black>
 				You can use your Browser, Email and other network Apps as you normally would.
 			</italic-black>
-		</p>
-		<p>
-			Your device originally requested <b>$originurl</b>
-			<br>
-			Click or tap Continue to go to there.
+
+			(Your device originally requested $originurl)
+			<hr>
+			Click or tap Continue to show the status of your account.
 		</p>
 		<form>
-			<input type=\"button\" VALUE=\"Continue\" onClick=\"location.href='$originurl'\" >
+			<input type=\"button\" VALUE=\"Continue\" onClick=\"location.href='$gatewayurl'\" >
 		</form>
 		<hr>
 	"
