@@ -1,9 +1,10 @@
 #!/bin/sh
 
 # Check if opennds is running
-ndspid=$(ps | grep opennds_cfg | awk -F ' ' 'NR==2 {print $1}')
+ndspid=$(pidof opennds)
+fwhook=$(uci -q get opennds.@opennds[0].fwhook_enabled)
 if [ ! -z $ndspid ]; then
-  if [ "$(uci -q get opennds.@opennds[0].fwhook_enabled)" = "1" ]; then
+  if [ "$fwhook" = "1" ]; then
     echo "fwhook restart request received - restarting " | logger -p "daemon.warn" -s -t "opennds[$ndspid]: "
     /etc/init.d/opennds restart
   fi
