@@ -517,7 +517,6 @@ get_iface_ip(const char ifname[], int ip6)
 	char addrbuf[INET6_ADDRSTRLEN] = {0};
 	char cmd[256] = {0};
 	char iptype[8] = {0};
-	int i;
 
 	if (ip6) {
 		snprintf(iptype, sizeof(iptype), "inet6");
@@ -525,25 +524,18 @@ get_iface_ip(const char ifname[], int ip6)
 		snprintf(iptype, sizeof(iptype), "inet");
  	}
 
-	snprintf(cmd, sizeof(cmd), "ip address | grep -A2 ': %s' | grep '%s ' | awk '$NF == \"%s\" {print $2}' | awk -F'/' 'NR==1 {printf $1}'",
+	snprintf(cmd, sizeof(cmd), "/usr/lib/opennds/libopennds.sh gatewayip \"%s\" \"%s\"",
 		ifname,
-		iptype,
-		ifname
+		iptype
 	);
 
 	debug(LOG_NOTICE, "Attempting to Bind to interface: %s", ifname);
 
-	for (i=0; i<10; i=i+1) {
-		execute_ret(addrbuf, sizeof(addrbuf), cmd);
-		if (is_addr(addrbuf) == 1) {
-			break;
-		} else {
-			debug(LOG_NOTICE, "Interface: %s is not yet ready - waiting...", ifname);
-			sleep(1);
-		}
+	if (execute_ret(addrbuf, sizeof(addrbuf), cmd) == 0) {
+		return safe_strdup(addrbuf);
+	} else {
+		return "error";
 	}
-
-	return safe_strdup(addrbuf);
 }
 
 char *
@@ -1051,7 +1043,7 @@ ndsctl_json_client(FILE *fp, const t_client *client, time_t now, char *indent)
 		(download_bytes / 1024)
 	);
 
-	fprintf(fp, "  %s\"download_session_avg\":\"%.2f\"\n",
+	fprintf(fp, "  %s\"download_session_avg\":\"%.2f\",\n",
 		indent,
 		(double)download_bytes / 125 / durationsecs
 	);
@@ -1061,7 +1053,7 @@ ndsctl_json_client(FILE *fp, const t_client *client, time_t now, char *indent)
 		(upload_bytes / 1024)
 	);
 
-	fprintf(fp, "  %s\"upload_session_avg\":\"%.2f\",\n",
+	fprintf(fp, "  %s\"upload_session_avg\":\"%.2f\"\n",
 		indent,
 		(double)upload_bytes / 125 / durationsecs
 	);
